import Thing from '../../../src/lib/thing'
import Serializers from '../../../src/lib/osapjs/utils/serializers'

export default class servo extends Thing {

  // calibrated-angle-bounds,
  private pulseBounds = [500, 2500] // pulse-width bounds
  private angleBounds = [0, 180]  // angular bounds

  async setRGB(r, g, b) {
    let datagram = new Uint8Array(3);
    datagram[0] = r * 255;
    datagram[1] = g * 255;
    datagram[2] = b * 255;
    await this.send("setRGB", datagram);
  }

  async setLED(state) {
    let datagram = new Uint8Array([state > 0]);
    await this.send("setLED", datagram);
  }

  async writeMicroseconds(us: number){
    try {
      us = Math.round(us);
      console.log(us)
      let datagram = new Uint8Array(2);
      Serializers.writeUint16(datagram, 0, us);
      await this.send("writeMicroseconds", datagram);
    } catch (err) {
      throw err
    }
  }


  async writeAngle(angle: number){
    try {
      // constrain to as-specified bounds,
      if (angle < this.angleBounds[0]) angle = this.angleBounds[0]
      if (angle > this.angleBounds[1]) angle = this.angleBounds[1]
      // interpolate with microseconds-bounds match,
      let interp = (angle - this.angleBounds[0]) / (this.angleBounds[1] - this.angleBounds[0])
      interp = interp * (this.pulseBounds[1] - this.pulseBounds[0]) + this.pulseBounds[0]
      // and sendy
      await this.writeMicroseconds(interp)
    } catch (err) {
      throw err
    }
  }
  async setcustomLED(r: number, g: number, b: number, brightness: number, numberLEDs: number, ) {
    let datagram = new Uint8Array(5);
    datagram[0] = r;
    datagram[1] = g;
    datagram[2] = b;
    datagram[3] = brightness;
    datagram[4] = numberLEDs;
    // to send data, we use 
    // `osap.send(name: string, targetPort: string, data: Uint8Array)`
    // ... and please use 'async' funcs with 'await' in front of 
    // network calls 
    // ... as you have probably figured out, "setRGB" here routes to 
    // the function defined around line 43 in the arduino example
    await this.send("setcustomLED", datagram);
  }


  setCalibration(_pulseBounds, _angleBounds){
    if (!Array.isArray(_pulseBounds) || !Array.isArray(_angleBounds)) {
      throw new Error(`input args for setCalibration are both arrays`)
    }
    this.pulseBounds = _pulseBounds
    this.angleBounds = _angleBounds
  }


  public api = [
    {
      name: "writeMicroseconds",
      args: [
        "us: 0 to 2^16"
      ]
    },
    {
      name: "writeAngle",
      args: [
        "angle: num"
      ]
    },
    {
      name: "setCalibration",
      args: [
        "pulseBounds: [minMicros, maxMicros]",
        "angleBounds: [minAngle, maxAngle]"
      ]
    },
    {
      name: "setcustomLED",
      args: [
        "red: 1 to 255",
        "green: 1 to 255",
        "blue: 1 to 255",
        "brightness: 1 to 255",
        "numberLEDs: 1 to any",
      ],
    },
  ]
}
